@@ -1,13 +1,15 @@
 #![allow(dead_code)]
 
-use crate::token::Token;
 use std::{borrow::Cow, fmt::Debug};
+
 use swc_atoms::JsWord;
 use swc_common::{
     errors::{DiagnosticBuilder, Handler},
     Span, Spanned,
 };
 
+use crate::token::Token;
+
 /// Note: this struct is 8 bytes.
 #[derive(Debug, Clone, PartialEq)]
 pub struct Error {
@@ -233,6 +235,7 @@ pub enum SyntaxError {
     TS2452,
     TS2483,
     TS2491,
+    TS2499,
     TS2703,
     TS4112,
     TSTypeAnnotationAfterAssign,
@@ -583,6 +586,9 @@ impl SyntaxError {
             SyntaxError::TS2491 => "The left-hand side of a 'for...in' statement cannot be a \
                                     destructuring pattern"
                 .into(),
+            SyntaxError::TS2499 => "An interface can only extend an identifier/qualified-name \
+                                    with optional type arguments."
+                .into(),
             SyntaxError::TS4112 => "This member cannot have an 'override' modifier because its \
                                     containing class does not extend another class."
                 .into(),
