@@ -1,9 +1,10 @@
-use super::{ident::MaybeOptionalIdentParser, *};
-use crate::{error::SyntaxError, lexer::TokenContext, parser::stmt::IsDirective, Tokens};
 use either::Either;
 use swc_atoms::js_word;
 use swc_common::{Spanned, SyntaxContext};
 
+use super::{ident::MaybeOptionalIdentParser, *};
+use crate::{error::SyntaxError, lexer::TokenContext, parser::stmt::IsDirective, Tokens};
+
 /// Parser for function expression and function declaration.
 impl<'a, I: Tokens> Parser<I> {
     pub(super) fn parse_async_fn_expr(&mut self) -> PResult<Box<Expr>> {
@@ -98,12 +99,7 @@ impl<'a, I: Tokens> Parser<I> {
             };
 
             let (mut super_class, mut super_type_params) = if eat!(p, "extends") {
-                let super_class = p.parse_lhs_expr().map(Some)?;
-                let super_type_params = if p.input.syntax().typescript() && is!(p, '<') {
-                    Some(p.parse_ts_type_args()?)
-                } else {
-                    None
-                };
+                let (super_class, super_type_params) = p.parse_super_class()?;
 
                 if p.syntax().typescript() && eat!(p, ',') {
                     let exprs = p.parse_ts_heritage_clause()?;
@@ -113,7 +109,7 @@ impl<'a, I: Tokens> Parser<I> {
                     }
                 }
 
-                (super_class, super_type_params)
+                (Some(super_class), super_type_params)
             } else {
                 (None, None)
             };
@@ -122,10 +118,7 @@ impl<'a, I: Tokens> Parser<I> {
             if eat!(p, "extends") {
                 p.emit_err(p.input.prev_span(), SyntaxError::TS1172);
 
-                p.parse_lhs_expr()?;
-                if p.input.syntax().typescript() && is!(p, '<') {
-                    p.parse_ts_type_args()?;
-                }
+                p.parse_super_class()?;
             };
 
             let implements = if p.input.syntax().typescript() && eat!(p, "implements") {
@@ -147,17 +140,12 @@ impl<'a, I: Tokens> Parser<I> {
             if p.input.syntax().typescript() && eat!(p, "extends") {
                 p.emit_err(p.input.prev_span(), SyntaxError::TS1173);
 
-                let sc = p.parse_lhs_expr()?;
-                let type_params = if p.input.syntax().typescript() && is!(p, '<') {
-                    p.parse_ts_type_args().map(Some)?
-                } else {
-                    None
-                };
+                let (sc, type_params) = p.parse_super_class()?;
 
                 if super_class.is_none() {
                     super_class = Some(sc);
-                    if let Some(tp) = type_params {
-                        super_type_params = Some(tp);
+                    if type_params.is_some() {
+                        super_type_params = type_params;
                     }
                 }
             }
@@ -188,6 +176,26 @@ impl<'a, I: Tokens> Parser<I> {
         })
     }
 
+    fn parse_super_class(&mut self) -> PResult<(Box<Expr>, Option<TsTypeParamInstantiation>)> {
+        let super_class = self.parse_lhs_expr()?;
+        match *super_class {
+            Expr::TsInstantiation(TsExprWithTypeArgs {
+                expr, type_args, ..
+            }) => Ok((expr, type_args)),
+            _ => {
+                // We still need to parse TS type arguments,
+                // because in some cases "super class" returned by `parse_lhs_expr`
+                // may not include `TsExprWithTypeArgs`
+                // but it's a super class with type params, for example, in JSX.
+                if self.syntax().typescript() && is!(self, '<') {
+                    Ok((super_class, self.parse_ts_type_args().map(Some)?))
+                } else {
+                    Ok((super_class, None))
+                }
+            }
+        }
+    }
+
     pub(super) fn parse_decorators(&mut self, allow_export: bool) -> PResult<Vec<Decorator>> {
         if !self.syntax().decorators() {
             return Ok(vec![]);
@@ -1314,6 +1322,7 @@ impl OutputType for Box<Expr> {
     fn finish_fn(_span: Span, ident: Option<Ident>, function: Function) -> Self {
         Box::new(Expr::Fn(FnExpr { ident, function }))
     }
+
     fn finish_class(_span: Span, ident: Option<Ident>, class: Class) -> Self {
         Box::new(Expr::Class(ClassExpr { ident, class }))
     }
@@ -1335,6 +1344,7 @@ impl OutputType for ExportDefaultDecl {
             decl: DefaultDecl::Fn(FnExpr { ident, function }),
         }
     }
+
     fn finish_class(span: Span, ident: Option<Ident>, class: Class) -> Self {
         ExportDefaultDecl {
             span,
@@ -1357,6 +1367,7 @@ impl OutputType for Decl {
             function,
         })
     }
+
     fn finish_class(_: Span, ident: Ident, class: Class) -> Self {
         Decl::Class(ClassDecl {
             declare: false,
@@ -1485,10 +1496,11 @@ struct MakeMethodArgs {
 mod tests {
     #![allow(unused)]
 
-    use super::*;
     use swc_common::DUMMY_SP as span;
     use swc_ecma_visit::assert_eq_ignore_span;
 
+    use super::*;
+
     fn lhs(s: &'static str) -> Box<Expr> {
         test_parser(s, Syntax::default(), |p| p.parse_lhs_expr())
     }
