@@ -1,9 +1,10 @@
-use super::*;
-use crate::{lexer::TokenContexts, parser::class_and_fn::IsSimpleParameterList};
 use either::Either;
 use swc_atoms::js_word;
 use swc_common::{Spanned, SyntaxContext};
 
+use super::*;
+use crate::{lexer::TokenContexts, parser::class_and_fn::IsSimpleParameterList};
+
 impl<I: Tokens> Parser<I> {
     /// `tsNextTokenCanFollowModifier`
     fn ts_next_token_can_follow_modifier(&mut self) -> PResult<bool> {
@@ -356,9 +357,16 @@ impl<I: Tokens> Parser<I> {
             .map(From::from)?
         };
 
+        let type_args = if is!(self, '<') {
+            Some(self.parse_ts_type_args()?)
+        } else {
+            None
+        };
+
         Ok(TsTypeQuery {
             span: span!(self, start),
             expr_name,
+            type_args,
         })
     }
 
@@ -498,6 +506,25 @@ impl<I: Tokens> Parser<I> {
         }
     }
 
+    pub(super) fn try_parse_type_args_of_ts_expr_with_type_args(
+        &mut self,
+    ) -> Option<TsTypeParamInstantiation> {
+        debug_assert!(self.input.syntax().typescript());
+
+        self.try_parse_ts(|p| {
+            let type_args = p.parse_ts_type_args()?;
+            if is_one_of!(
+                p, ',', '.', '?', ')', ']', ':', '&', '|', '^', '}', "??", "==", "===", "!=",
+                "!==", "&&", "||"
+            ) || is_exact!(p, ';')
+            {
+                Ok(Some(type_args))
+            } else {
+                Ok(None)
+            }
+        })
+    }
+
     /// `tsTryParse`
     pub(super) fn try_parse_ts<T, F>(&mut self, op: F) -> Option<T>
     where
@@ -898,31 +925,41 @@ impl<I: Tokens> Parser<I> {
         debug_assert!(self.input.syntax().typescript());
 
         self.parse_ts_delimited_list(ParsingContext::HeritageClauseElement, |p| {
-            p.parse_expr_with_type_args()
+            p.parse_ts_heritage_clause_element()
         })
     }
 
-    /// `tsParseExpressionWithTypeArguments`
-    fn parse_expr_with_type_args(&mut self) -> PResult<TsExprWithTypeArgs> {
+    fn parse_ts_heritage_clause_element(&mut self) -> PResult<TsExprWithTypeArgs> {
         debug_assert!(self.input.syntax().typescript());
 
         let start = cur_pos!(self);
         // Note: TS uses parseLeftHandSideExpressionOrHigher,
         // then has grammar errors later if it's not an EntityName.
 
-        let expr = self.parse_ts_entity_name(/* allow_reserved_words */ false)?;
-        let type_args = if is!(self, '<') {
-            Some(self.parse_ts_type_args()?)
-        } else {
-            None
-        };
+        let ident = Box::new(Expr::Ident(self.parse_ident_name()?));
+        let expr = self.parse_subscripts(Callee::Expr(ident), true, true)?;
+        if !matches!(&*expr, Expr::Ident(..) | Expr::Member(..)) {
+            self.emit_err(span!(self, start), SyntaxError::TS2499);
+        }
 
-        Ok(TsExprWithTypeArgs {
-            span: span!(self, start),
-            expr,
-            type_args,
-        })
+        match *expr {
+            Expr::TsInstantiation(ts_expr_with_type_args) => Ok(ts_expr_with_type_args),
+            _ => {
+                let type_args = if is!(self, '<') {
+                    Some(self.parse_ts_type_args()?)
+                } else {
+                    None
+                };
+
+                Ok(TsExprWithTypeArgs {
+                    span: span!(self, start),
+                    expr,
+                    type_args,
+                })
+            }
+        }
     }
+
     /// `tsParseInterfaceDeclaration`
     pub(super) fn parse_ts_interface_decl(&mut self, start: BytePos) -> PResult<TsInterfaceDecl> {
         debug_assert!(self.input.syntax().typescript());
@@ -2658,13 +2695,14 @@ fn make_decl_declare(mut decl: Decl) -> Decl {
 
 #[cfg(test)]
 mod tests {
-    use crate::{
-        lexer::Lexer, test_parser, token::*, Capturing, EsVersion, Parser, Syntax, TsConfig,
-    };
     use swc_common::DUMMY_SP;
     use swc_ecma_ast::*;
     use swc_ecma_visit::assert_eq_ignore_span;
 
+    use crate::{
+        lexer::Lexer, test_parser, token::*, Capturing, EsVersion, Parser, Syntax, TsConfig,
+    };
+
     #[test]
     fn issue_708_1() {
         let actual = test_parser(
